# Generated by roxygen2: do not edit by hand

export(CF)
export(CFbuilder)
export(kclosestitems)
export(topkitems)
export(topkusers)
importFrom(Matrix,Matrix)
importFrom(Matrix,cbind2)
importFrom(Matrix,rbind2)
importFrom(Matrix,sparseMatrix)
importFrom(R6,R6Class)
importFrom(utils,head)
importFrom(utils,setTxtProgressBar)
importFrom(utils,txtProgressBar)
